/* $Id: mn_tool.c,v 1.55 2001/07/11 15:12:45 jm Exp $
 * Mobile Node Configuration Tool implemetation file
 *
 * Dynamic hierarchial IP tunnel
 * Copyright (C) 1998-2001, Dynamics group
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See README and COPYING for
 * more details.
 * 
 * Additional modifications Copyright 2010 Juho Paaso, Aalto 
 * University / Department of Communication and Networking"
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#ifndef __USE_BSD
#define __USE_BSD /* Add strncasecmp() etc. */
#endif
#include <string.h>
#include <sys/time.h>
#include <unistd.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#ifdef DYN_TARGET_LINUX
#include <values.h>
#endif
#include <time.h>

#ifdef DYN_TARGET_WINDOWS
#include "windows_extra.h"
#endif

#include "dyn_mnlib.h"
#include "support_tool.h"
#include "util.h"

#define BUFLEN 1024
#define PATH_SIZE  100
#define DEFAULT_PATH SOCKETDIR "/dynamics_mn_admin"

#define HELP_CAREOF "Usage\n\tcareof\n" \
"Description\n\tGet current careof address for the mobile.\n"

#define HELP_PRINT_TUNNELS "Usage\n\tprint_tunnels\n" \
"Description\n\tPrint all MR tunnels' source devices and destination CoAs.\n"

#define HELP_SWITCH_TNL_IFACE "Usage\n\tswitch_tnl_iface <tnl dst HoA> <nw iface name>\n" \
"Description\n\tSwitch tunnel to go through another networking interface.\n" \
"Example: switch_tnl_iface 172.22.30.254 eth1\n"

#define HELP_STATUS "Usage\n\tstatus [interval]\n" \
"Description\n\tRetrieve agents status information.\n"\
"\tIf interval is specified, command is run in monitoring mode and "\
"interval specifies time between successive calls in seconds."\
"To stop monitoring press CTRL-C.\n"

#define HELP_CONNECT "Usage\n\ttunnel [mode]\n\tconnect [mode]\n" \
"Description\n\tStart tunneling packets using Dynamics tunneling system.\n" \
"Modes for tunneling are:\n"\
"\t\t 0 - no tunnel\n"\
"\t\t 1 - full tunnel\n"\
"\t\t 2 - triangle tunnel\n"\
"\t\t HA - connect directly to the Home Agent with mode 1\n"\
"Default mode for tunneling is full tunnel.\n"

#define HELP_DISC "Usage\n\tdisconnect\n" \
"Description\n\tDisconnect from tunneling system.\n"
#define HELP_LOCUPD "Usage\n\tupdate [block] <new-address>" \
" | update [block] <interface> | update [block]\n" \
"Description\n\tPerform location update and " \
"start using [address <new-address> |\n\taddress of <interface> | address of" \
" default interface] as co-located address.\n"
#define HELP_CONFIRM "Usage\n\tconfirm\n" \
"Description\n\tSend tunnel refreshing message."
#define HELP_PATH "Usage\n\tpath <api path>\n" \
"Description\n\tSet path to agent interface socket.\n" \
"Example: path /etc/dynamics_mn_admin\n"
#define HELP_FORCEFA "Usage\n\tforce [<fa-addr>]\n" \
"Description\n\tForces MN to select specified FA for tunneling.\n" \
"\tIf set to 0.0.0.0 (or no argument), force is\n\tremoved.\n" \
"Example: force 10.2.3.4\n"
#define HELP_LIST "Usage\n\tlist\n" \
"Description\n\tLists heard FAs\n"
#define HELP_SHOW "Usage\n\tshow <fa-addr> <interface>\n" \
"Description\n\tShows detailed information about the FA\n"
#define HELP_WIRELESS "Usage\n\twireless <command> [<options> ..]\n" \
"\nDescription\n\tWireless extensions api interface. Command(s):\n\n\t"\
"channel [<interface> <channel>]\n\n\tDescription\n\t\tWith "\
"no arguments current"\
" channel(s) are shown.\n\t\t<interface> is the wireless interface and"\
" <channel> is\n\t\tthe channel number we want to set the"\
" wireless\n\t\tinterface to use.\n"\
"\tExample: wireless channel eth0 3\n"
#define HELP_RESCAN "Usage\n\t rescan\n"\
"\nDescription\n\tDetects new and removed interfaces.\n"
#define HELP_REGISTER "Usage\n\tregister <device info socket path>\n"
#define HELP_POLICY "Usage\n\tpolicy [<policy> on|off]\n"
#define HELP_MONITOR "Usage\n\tmonitor [<variable> [<value>]]\n"

struct tool_cmd_completion cmd_wireless_completion[] =
  {
    { "channel", NULL },
    { NULL, NULL }
  };

struct tool_cmd_completion cmd_boolean_completion[] =
  {
    { "on", NULL },
    { "off", NULL },
    { NULL, NULL }
  };

struct tool_cmd_completion cmd_policy_completion[] =
  { { "Early-expire", cmd_boolean_completion },
    { "Newest-FA", cmd_boolean_completion },
    { "Eager-switching", cmd_boolean_completion },
    { "Newest-ADV", cmd_boolean_completion },
    { NULL, NULL }
  };




void cmd_getcoaddr(int argc, char *argv[]);
void cmd_getmode(int argc, char *argv[]);
void cmd_print_tunnels(int argc, char *argv[]);
void cmd_switch_tnl_iface(int argc, char *argv[]);
void cmd_getstatus(int argc, char *argv[]);
void cmd_connect(int argc, char *argv[]);
void cmd_disconnect(int argc, char *argv[]);
void cmd_locupd(int argc, char *argv[]);
void cmd_confirm(int argc, char *argv[]);
void cmd_path(int argc, char *argv[]);
void cmd_force(int argc, char *argv[]);
void cmd_list(int argc, char *argv[]);
void cmd_show(int argc, char *argv[]);
void cmd_wireless(int argc, char *argv[]);
void cmd_rescan(int argc, char *argv[]);
void cmd_register(int argc, char *argv[]);
void cmd_policy(int argc, char *argv[]);
void cmd_monitor(int argc, char *argv[]);
extern int interactive;

#define MAX_NUM_FA 64

int main(int argc, char *argv[])
{
	int i;
	char agent_path[PATH_SIZE];

	dynamics_strlcpy(agent_path, DEFAULT_PATH, PATH_SIZE);

	tool_add_command("update", 3, cmd_locupd, HELP_LOCUPD, NULL);
	tool_add_command("tunnel", 3, cmd_connect, HELP_CONNECT, NULL);
	tool_add_command("connect", 3, cmd_connect, HELP_CONNECT, NULL);
	tool_add_command("status", 1, cmd_getstatus, HELP_STATUS, NULL);
	tool_add_command("disconnect", 3, cmd_disconnect, HELP_DISC, NULL);
	tool_add_command("careof", 2, cmd_getcoaddr, HELP_CAREOF, NULL);
	tool_add_command("confirm", 4, cmd_confirm, HELP_CONFIRM, NULL);
	tool_add_command("path", 4, cmd_path, HELP_PATH, NULL);
	tool_add_command("force", 5, cmd_force, HELP_FORCEFA, NULL);
	tool_add_command("list", 1, cmd_list, HELP_LIST, NULL);
	tool_add_command("show", 2, cmd_show, HELP_SHOW, NULL);
	tool_add_command("wireless", 1, cmd_wireless, HELP_WIRELESS,
			 cmd_wireless_completion);
	tool_add_command("rescan", 3, cmd_rescan, HELP_RESCAN, NULL);
	tool_add_command("register", 3, cmd_register, HELP_REGISTER, NULL);
	tool_add_command("policy", 1, cmd_policy, HELP_POLICY,
			 cmd_policy_completion);
	tool_add_command("monitor", 1, cmd_monitor, HELP_MONITOR, NULL);
	tool_add_command("print_tunnels", 1, cmd_print_tunnels, HELP_PRINT_TUNNELS, NULL);
	tool_add_command("switch_tnl_iface", 1, cmd_switch_tnl_iface, HELP_SWITCH_TNL_IFACE, NULL);
	
	i = parse_long_options_tools(argc, argv,
				     "Dynamics Mobile Agent Control Tool",
				     agent_path, PATH_SIZE,
				     PACKAGE, VERSION);

	dynamics_mn_init(agent_path);

	if (interactive) {
		printf("Dynamics Mobile Agent Control Tool v%s\n", VERSION);
		printf("Using agent path \"%s\" \n", agent_path);
	}
	return tool_main(argc - i , argv + i);
}

/***********************************************************************
 *              COMMAND HANDLERS                                       *
 * These functions are callback functions, which tool_main calls       *
 * when user executes a command. Commands arguments are passed         *
 * in parameters. argc specifies count of aruments and argv is pointer *
 * to string array containing the actual argument strings.             *
 * first argument is always commands invocation. It may be an          *
 * abbreviation.                                                       *
 ***********************************************************************/

void cmd_getcoaddr(int argc, char *argv[])
{
	struct in_addr addr;
	int r;

	r = dynamics_mn_get_care_of_addr(&addr, -1);

	if (r == API_SUCCESS)
		printf("Care-of address: %s\n", inet_ntoa(addr));
	else {
		printf("Call failed: %s\n", dynamics_mn_get_error_string(r));
	}

}

void cmd_getmode(int argc, char *argv[])
{
	int r,i;
	r = dynamics_mn_get_tunneling_mode(&i, -1);
	if (r == API_SUCCESS) {
		printf("Tunneling mode: ");
		switch (i) {
		case API_TUNNEL_NONE:
			printf("none");
			break;
		case API_TUNNEL_FULL:
			printf("full");
			break;
		case API_TUNNEL_TRIANGLE:
			printf("triangle");
			break;
		case API_TUNNEL_FULL_HA:
			printf("full directly to HA");
			break;
		default:
			printf("unknown mode %i", i);
		}
		printf("\n");
	} else
		printf("Call failed: %s\n", dynamics_mn_get_error_string(r));
}
/* cmd_print_tunnels: Print tunnels' source devices and destination CoAs */
void cmd_print_tunnels(int argc, char *argv[])
{
	struct mr_tunnels mr_tunnels;

	int r, i;
	
	r = mn_get_tnl_info(&mr_tunnels, 1);
	
	if (r == API_SUCCESS) {
		printf("\nMR's tunnels:\n");
		for (i = 0; i < mr_tunnels.tnl_count; i++)
		{
			printf("\n\tname     \t%s\n", mr_tunnels.tunnels[i].tun_dev_name);
			printf("\tdevice   \t%s\n", mr_tunnels.tunnels[i].eth_dev_name);
			
			/* Print dst HoA for CRs and "Home Agent" for HA tunnel */
			if (mr_tunnels.tunnels[i].is_ha_tunnel)
				printf("\tdst HoA  \tHome Agent\n");
			else
				printf("\tdst HoA  \t%s\n", inet_ntoa(mr_tunnels.tunnels[i].dst_hoa));
			
			printf("\tsrc CoA  \t%s:%d\n", 
				inet_ntoa(mr_tunnels.tunnels[i].src_coa),
				mr_tunnels.tunnels[i].src_port);
			printf("\tdst CoA  \t%s:%d\n", 
				inet_ntoa(mr_tunnels.tunnels[i].dst_coa),
				mr_tunnels.tunnels[i].dst_port);
			printf("\tRTT      \t%d\n", mr_tunnels.tunnels[i].rtt);
		}
		printf("\n");
	} else
		printf("Call failed: %s\n", dynamics_mn_get_error_string(r));
}
/* cmd_switch_tnl_iface: Switch a tunnel to go through another nw iface */
void cmd_switch_tnl_iface(int argc, char *argv[])
{
	int r;
	struct switch_tnl_iface container;
	
	if (argc == 3) {
		inet_aton(argv[1], &container.dst_hoa);
		memcpy(container.new_iface, argv[2], IFNAMSIZ);
		printf("\nSwitching the tunnel of HoA %s to go through interface %s\n", 
			inet_ntoa(container.dst_hoa), container.new_iface);
		r = mn_switch_tnl_iface(&container, 1);
		
		if (r == API_SUCCESS) {
			printf("Request ok\n");
		} else {
			printf("Error in switching!\n");
			
			if (r == API_NOT_PERMITTED)
				printf(" - invalid iface\n");
			else if (r == API_ILLEGAL_PARAMETERS)
				printf(" - invalid HoA\n");
			else
				printf(" - unknown reason\n");
		}
	} else {
		printf("\nTwo parametres needed!\n\n%s", HELP_SWITCH_TNL_IFACE);
	}
	printf("\n");
}
void cmd_getstatus(int argc, char *argv[])
{
	int r;
	struct dynamics_mobile_status status;
	time_t current_time;

	if (argc == 2 &&
	    (r = atoi(argv[1])) > 0) {
		tool_monitor(argv[0], r);
	}

	r = dynamics_mn_get_status(&status, -1);
	if (r == API_SUCCESS) {
		current_time = time(NULL);
		printf("Mobile status:\n");
		printf("\tstate     \t%s\n", status.state_str);
		printf("\tlocal addr\t%s\n", inet_ntoa(status.local_addr));
		printf("\tco-addr   \t%s\n", inet_ntoa(status.co_addr));
		printf("\tFA-addr   \t%s\n", inet_ntoa(status.fa_addr));
		printf("\tHA-addr   \t%s\n", inet_ntoa(status.ha_addr));
		printf("\tHome addr \t%s\n", inet_ntoa(status.home_addr));
		if (status.tunnel_up) {
			printf("\ttunnel is \tup\n");
			printf("\tlifetime left\t%ds\n", status.lifetime);
		} else printf("\ttunnel is \tdown\n");
		printf("\ttunneling mode\t%s\n", 
		       dynamics_mn_tunnel_mode_desc(status.tunnel_mode));
		if (status.last_request_sent != 0)
			printf("\tlast request\t%ds ago; %s ",
			       (int)(current_time - status.last_request_sent),
			       ctime(&status.last_request_sent));
		if (status.last_reply_rcvd != 0) {
			printf("\tlast reply\t%ds ago; %s",
			       (int)(current_time - status.last_reply_rcvd),
			       ctime(&status.last_reply_rcvd));
			
			printf("\treply code\t%i - %s\n",
			       status.last_reply_code,
			       reply_code_str(status.last_reply_code));
		}
		if (status.info_str[0] != '\0') {
			status.info_str[API_INFO_LEN] = '\0';
			printf("\tinfo text\t%s\n", status.info_str);
		}
		if (status.warn_str[0] != '\0') {
			status.warn_str[API_INFO_LEN] = '\0';
			printf("\tlast warning\t%s\n", status.warn_str);
		}
		printf("\tactive devices\t%i\n", status.device_count);
		printf("\tdiscarded msgs\t%li\n", status.discarded_msgs);
	} else
		printf("Call failed: %s\n", dynamics_mn_get_error_string(r));
}

void cmd_connect(int argc, char *argv[])
{
	int r, mode;
	mode = API_TUNNEL_FULL;
	if (argc == 2) {
		r = atoi(argv[1]);
		if (r > 0)
			mode = r;
		else if (strcmp("HA", argv[1]) == 0)
			mode = API_TUNNEL_FULL_HA;
	}
	r = dynamics_mn_connect(mode, -1);
	if (r == API_SUCCESS)
		printf("Connect success.\n");
	else
		printf("Call failed: %s\n", dynamics_mn_get_error_string(r));
}

void cmd_disconnect(int argc, char *argv[])
{
	int r;
	r = dynamics_mn_disconnect(-1);
	if (r == API_SUCCESS)
		printf("Disconnected\n");
	else
		printf("Call failed: %s\n", dynamics_mn_get_error_string(r));
}

void cmd_locupd(int argc, char *argv[])
{
	int r, pos, block;
	struct in_addr addr;

	pos = 1;
	block = 0;

	if (argc > 1 && strcmp(argv[1], "block") == 0) {
		block = 1;
		pos = 2;
		printf("Blocking call\n");
	} else if (argc > 2) {
		printf("Invalid arguments. Usage example "
		       "'update 10.11.12.13'\n");
		return;
	}

	if (argc <= pos) {
		printf("Trying to use the default interface address.\n");
		if (!block)
			r = dynamics_mn_update_location_dev(NULL, -1);
		else
			r = dynamics_mn_update_location_dev_block(NULL, -1);
	} else if (inet_aton(argv[pos], &addr) != 0) {
		if (!block)
			r = dynamics_mn_update_location(&addr, -1);
		else
			r = dynamics_mn_update_location_block(&addr, -1);
	} else {
		printf("Trying to use the address of the interface %s\n",
		       argv[pos]);
		if (!block)
			r = dynamics_mn_update_location_dev(argv[pos], -1);
		else
			r = dynamics_mn_update_location_dev_block(argv[pos],
								  -1);
	}

	if (r == API_SUCCESS)
		printf("Location updated.\n");
	else
		printf("Call failed: %s\n", dynamics_mn_get_error_string(r));
}

void cmd_confirm(int argc, char *argv[])
{
	int r;
	r = dynamics_mn_confirm(-1);
	if (r == API_SUCCESS)
		printf("Confirmed.\n");
	else
		printf("Call failed: %s\n", dynamics_mn_get_error_string(r));
}

void cmd_path(int argc, char *argv[])
{
	char *p;

	if (argc < 2) {
		p = dynamics_mn_get_agent_path();
		if (p) printf("Current agent path is \"%s\".\n", p); 
		return;
	} else if (argc > 2) {
		printf("Invalid arguments. Usage 'path <path>'.\n");
		return;
	}

	if (dynamics_mn_init(argv[1]) == API_SUCCESS) {
		printf("Using agent '%s'.\n", argv[1]);
	} else printf("Failed.\n");
}

void cmd_force(int argc, char *argv[])
{
	int r;
	struct in_addr addr;

	if (argc >= 2 &&
	    inet_aton(argv[1], &addr) == 0) {
		printf("Invalid arguments. Usage example 'force 10.1.2.3'\n");
		return;
	}

	if (argc == 1)
		addr.s_addr = 0;

	r = dynamics_mn_force_fa(&addr, -1);
	if (r == API_SUCCESS)
		printf("FA selection forced to %s.\n", inet_ntoa(addr));
	else
		printf("Call failed: %s\n", dynamics_mn_get_error_string(r));
}

void cmd_list(int argc, char *argv[])
{
	int r, c;
	struct dynamics_mobile_fa_list data[MAX_NUM_FA];
	struct timeval tstamp;

	if (argc == 2 &&
	    (r = atoi(argv[1])) > 0) {
		tool_monitor(argv[0], r);
	}

	c = MAX_NUM_FA;
	r = dynamics_mn_get_fa_list(&c, data, -1);
	gettimeofday(&tstamp, NULL);
	if (r == API_SUCCESS) {
		printf("List of heard mobility agents:\n");
		for (r = 0; r < c; r++) {
			printf("%-16s %-5s ",
			       inet_ntoa(data[r].addr),
			       data[r].interface);
			if (data[r].priority != -2 && 
			    data[r].priority != MAXINT)
				printf("prio %2d (-%3d%%), ", 
				       data[r].priority, data[r].reduce);
			if (data[r].priority == MAXINT)
				printf("prio MAX (-%3d%%), ",
				       data[r].reduce);
			printf("age %2ds", (int)(tstamp.tv_sec - 
						 data[r].last));
			if (data[r].quality_avg >= 0)
				printf(", qual %2d%%", data[r].quality_avg);
			if (data[r].flags & API_MA_FLAGS_FA)
				printf(" FA");
			if (data[r].flags & API_MA_FLAGS_HA)
				printf(" HA");
			if (data[r].flags & API_MA_FLAGS_BUSY)
				printf(" BUSY");
			if (data[r].flags & API_MA_FLAGS_DYNAMICS)
				printf(" DYN");
			if (data[r].in_use == 1)
				printf(" IN-USE");
			if (data[r].current_adv == 1)
				printf(" CURRENT");
			printf("\n");
		}
	} else {
		printf("Call failed: %s\n", dynamics_mn_get_error_string(r));
	}
}

void cmd_show(int argc, char *argv[])
{
	int r;
	struct dynamics_mobile_fa_info info;
	struct timeval tstamp;

	if (argc < 2) {
		printf("Invalid arguments. Usage 'show <FA-addr> "
		       "[<interface>]'.\n");
		return;
	}

	if (inet_aton(argv[1], &info.list.addr) == 0) {
		printf("Invalid <FA-addr> argument\n");
		return;
	}

	gettimeofday(&tstamp, NULL);
	if (argc >= 3) {
		if (strlen(argv[2]) > IFNAMSIZ - 1) {
			printf("Invalid <interface> argument. "
			       "Too long (max %d).\n", IFNAMSIZ - 1);
			return;
		}
		dynamics_strlcpy(info.list.interface, argv[2], IFNAMSIZ);
	} else 
		info.list.interface[0] = '\0';


	r = dynamics_mn_get_fa_info(&info, -1);
	if (r == API_SUCCESS) {
		printf("FA                 %s",
		       inet_ntoa(info.list.addr));
		if (info.list.in_use)
			printf(" (IN-USE)");
		printf("\nInterface          %s\n",
		       info.list.interface);
		printf("Interface index    %d\n",
		       info.ifindex);
		printf("Priority           %d\n",
		       info.list.priority);
		if (info.list.quality_avg > 0)
			printf("Link quality       %2d%%\n",
			       info.list.quality_avg);
		printf("Last adv.          %ds (expires in %ds)\n",
		       (int)(info.list.last - tstamp.tv_sec),
		       (int)(info.expire - tstamp.tv_sec));
		if (info.nai[0] != '\0')
			printf("NAI                %s\n", info.nai);
	} else {
		printf("Call failed: %s\n", dynamics_mn_get_error_string(r));
	}
}


static int match(char *cmp1, char *cmp2)
{
	int len = strlen(cmp1);
	if (len > strlen(cmp2))
		return -1;
	return memcmp(cmp1, cmp2, len);
}

static void wireless_set_channel(int argc, char *argv[]) 
{
	int channel = -1;
	char *ifname = NULL;
	int r;

	ifname = argv[0];
	channel = atoi(argv[1]);
	if (channel < 0 || channel > 14) {
		printf("Invalid channel (%d). Use 1 - 14\n", channel);
		return;
	}
	r = dynamics_mn_iw_set_ch(channel, ifname, -1);

	if (r == API_SUCCESS) {
		printf("Channel changed (%s, ch %d)\n", 
		       ifname, channel);
	} else {
		printf("Call failed: %s\n", dynamics_mn_get_error_string(r));
	}
}

static void wireless_get_channel(void)
{
#define MAX_NUM_IF 5
	struct dynamics_mn_iw_ch_info info[MAX_NUM_IF];
	int r, i, c = MAX_NUM_IF;

	memset(info, 0, sizeof(info));
	r = dynamics_mn_iw_get_ch(&c, info, -1);

	if (r == API_SUCCESS) {
		for (i = 0; i < c; i++)
			if (info[i].ifname != NULL) {
				printf("Interface: %s, Channel: %d, AP: %s\n",
				       info[i].ifname, info[i].channel,
				       ether_hwtoa((unsigned char *)
						   info[i].ap_hwaddr));
			} else
				break;
	} else {
		printf("Call failed: %s\n", dynamics_mn_get_error_string(r));
	}
}

void cmd_wireless(int argc, char *argv[])
{
	char short_usage[]="wireless <command> [<options> ..]\n"
		"command(s):\n\tchannel [<interface> <channel>]\n";

	if (argc < 2) {
		printf("%s", short_usage);
	} else {
		argc--; argv++;
		/* Branch according to the command */
		if (!match(argv[0], "channel")) {
			/* Set/get channel */
			argc--; argv++;
			if (argc > 1)
				wireless_set_channel(argc, argv);
			else
				wireless_get_channel();
			return;
		} else if (0) {
			/* Add other wireless commands here */
			return;
		} else {
			/* default */
			printf("Unknown command '%s'. Try 'help wireless'.\n"
			       "%s", argv[0], short_usage);
			return;
		}
	}
}

void cmd_rescan(int argc, char *argv[])
{
	int r;

	r = dynamics_mn_rescan(-1);
	if (r != API_SUCCESS)
		printf("Call failed: %s\n", dynamics_mn_get_error_string(r));
}

void cmd_register(int argc, char *argv[])
{
	if (argc != 2) {
		printf("Invalid arguments. Usage 'register <path>'.\n");
		return;
	}

	if (dynamics_mn_register_dev_info_socket(argv[1], -1) == API_SUCCESS)
		printf("Succeeded.\n");
	else
		printf("Failed.\n");
}

void cmd_policy(int argc, char *argv[])
{
	int r;
	char buffer[BUFLEN];

	if (argc == 1) {
		memset(buffer, 0, sizeof(buffer));
		if (dynamics_mn_get_policy(buffer, BUFLEN, -1) == API_SUCCESS)
			printf(buffer);
		else
			printf("Get policy failed.\n");
		return;
	}

	if (argc != 3) {
		printf("Not enough arguments. "
		       "Usage 'policy <policy> on|off'.\n");
		return;
	}

	if (strcmp("on", argv[2]) == 0)
		r = dynamics_mn_policy_on(argv[1], -1);
	else if (strcmp("off", argv[2]) == 0)
		r = dynamics_mn_policy_off(argv[1], -1);
	else {
		printf("Use \"on\" or \"off\". "
		       "Usage 'policy <policy> on|off'.\n");
		return;
	}

	if (r == API_SUCCESS)
		printf("Set policy succeeded.\n");
	else
		printf("Set policy failed.\n");
}


void cmd_monitor(int argc, char *argv[])
{
	int r, value, varlen;
	char buffer[BUFLEN];
	
	memset(buffer, 0, sizeof(buffer));
	if (argc == 1) {
		r = dynamics_mn_get_mon_conf(buffer, BUFLEN, -1);
		if (r == API_SUCCESS)
			printf(buffer);
		else
			printf("Get monitor state failed.\n%s\n",
			       dynamics_mn_get_error_string(r));
		return;
	}

	if (argc == 2) {
		varlen = strlen(argv[1]);
		r = dynamics_mn_get_mon_conf_var(argv[1], varlen, buffer,
						 BUFLEN, -1);
		if (r == API_SUCCESS)
			printf(buffer);
		else
			printf("Get monitor state variable failed.\n%s\n",
			        dynamics_mn_get_error_string(r));
		return;
	}

	if (argc > 3) {
		printf("Too many arguments.\n"
		       "Usage 'monitor [<variable> [<value>]]'\n");
		return;
	}

	value = atoi(argv[2]);
	varlen = strlen(argv[1]);
	r = dynamics_mn_set_mon_conf_var(argv[1], varlen, value, -1);
	if (r == API_SUCCESS)
		printf("Set monitor state variable succeeded.\n");
	else
		printf("Set monitor state variable failed.\n%s\n",
		       dynamics_mn_get_error_string(r));
}

